@@ -4,18 +4,20 @@
 """
 from matplotlib import pyplot as plt
 
-from leap_ec import ops
-from leap_ec.context import context
+from leap_ec import Individual, Representation
+from leap_ec import ops, probe
 from leap_ec.algorithm import generational_ea
-from leap_ec.real_rep.ops import mutate_gaussian
-from leap_ec.representation import Representation
-from leap_ec.individual import Individual
 from leap_ec.problem import FunctionProblem
-from leap_ec.decoder import IdentityDecoder
-from leap_ec.real_rep.initializers import create_real_vector
+from leap_ec.real_rep import create_real_vector
+from leap_ec.real_rep.ops import mutate_gaussian
 
+
+##############################
+# Function ea_solve()
+##############################
 def ea_solve(function, bounds, generations=100, pop_size=2,
-             mutation_std=1.0, maximize=False, viz=False, viz_ylim=(0, 1)):
+             mutation_std=1.0, maximize=False, viz=False, viz_ylim=(0, 1),
+             hard_bounds=True):
     """Provides a simple, top-level interfact that optimizes a real-valued
     function using a simple generational EA.
 
@@ -30,11 +32,16 @@ def ea_solve(function, bounds, generations=100, pop_size=2,
     :param float mutation_std: the width of the mutation distribution
     :param bool maximize: whether to maximize the function (else minimize)
     :param bool viz: whether to display a live best-of-generation plot
+    :param bool hard_bounds: if True, bounds are enforced at all times during
+        evolution; otherwise they are only used to initialize the population.
 
     :param (float, float) viz_ylim: initial bounds to use of the plots
         vertical axis
 
-    >>> from leap_ec import simple
+    The basic call includes instrumentation that prints the best-so-far fitness
+    value of each generation to stdout:
+
+    >>> from leap_ec.simple import ea_solve
     >>> ea_solve(sum, bounds=[(0, 1)]*5) # doctest:+ELLIPSIS
     generation, bsf
     0, ...
@@ -42,28 +49,50 @@ def ea_solve(function, bounds, generations=100, pop_size=2,
     ...
     100, ...
     [..., ..., ..., ..., ...]
+
+    When `viz=True`, a live BSF plot will also display:
+
+    >>> ea_solve(sum, bounds=[(0, 1)]*5, viz=True) # doctest:+ELLIPSIS
+    generation, bsf
+    0, ...
+    1, ...
+    ...
+    100, ...
+    [..., ..., ..., ..., ...]
+
+    .. plot::
+
+        from leap_ec.simple import ea_solve
+        ea_solve(sum, bounds=[(0, 1)]*5, viz=True)
+
     """
 
+    if hard_bounds:
+        mutation_op = mutate_gaussian(std=mutation_std, hard_bounds=bounds,
+                        expected_num_mutations='isotropic')
+    else:
+        mutation_op = mutate_gaussian(std=mutation_std,
+                        expected_num_mutations='isotropic')
+
     pipeline = [
         ops.tournament_selection,
         ops.clone,
-        mutate_gaussian(std=mutation_std),
+        mutation_op,
         ops.uniform_crossover(p_swap=0.4),
         ops.evaluate,
         ops.pool(size=pop_size)
     ]
 
     if viz:
-        plot_probe = probe.PopulationPlotProbe(
-            context, ylim=viz_ylim, ax=plt.gca())
+        plot_probe = probe.FitnessPlotProbe(ylim=viz_ylim, ax=plt.gca())
         pipeline.append(plot_probe)
 
-    ea = generational_ea(generations=generations, pop_size=pop_size,
+    ea = generational_ea(max_generations=generations,
+                         pop_size=pop_size,
                          problem=FunctionProblem(function, maximize),
 
                          representation=Representation(
                              individual_cls=Individual,
-                             decoder=IdentityDecoder(),
                              initialize=create_real_vector(bounds=bounds)
                          ),
 
