@@ -1,18 +1,19 @@
 """
     Unit tests for cloning
 """
-from leap_ec import core
-from leap_ec import binary_problems
-from leap_ec import ops
+from leap_ec.individual import Individual
+from leap_ec.decoder import IdentityDecoder
+from leap_ec.binary_rep.problems import MaxOnes
+import leap_ec.ops as ops
 
 
 def test_clone():
     # We need an encoder and problem to ensure those float across during
     # clones.
-    decoder = core.IdentityDecoder()
-    problem = binary_problems.MaxOnes()
+    decoder = IdentityDecoder()
+    problem = MaxOnes()
 
-    original = core.Individual([1, 1], decoder=decoder, problem=problem)
+    original = Individual([1, 1], decoder=decoder, problem=problem)
 
     cloned = next(ops.clone(iter([original])))
 
@@ -24,7 +25,3 @@ def test_clone():
     assert original.decoder == cloned.decoder
     assert original.problem == cloned.problem
     assert original.__dict__ == cloned.__dict__
-
-
-
-
